---------------------------
-- lets simplify a bit..
---------------------------
local gf = Glance.Functions
local gv = Glance.Variables
local ga = Glance.Arrays
local gb = Glance.Buttons

---------------------------
-- create the button
---------------------------
gf.AddButton("Mounts","RIGHT")
local btn = gb.Mounts
btn.text              = "      "
btn.enabled           = true
btn.texture.normal    = "Interface\\AddOns\\Glance_Mounts\\mount.tga"
btn.events            = {"COMPANION_LEARNED"}
btn.update            = true
btn.click             = true
btn.tooltip           = true
btn.menu              = true
btn.options           = true
btn.onload            = true
btn.save.perAccount = {["Mounts"] = {}}
local spa = btn.save.perAccount

---------------------------
-- shortcuts
---------------------------
local spa = btn.save.perAccount
local HEX = ga.colors.HEX
local tooltip = gf.Tooltip

---------------------------
-- not available till lvl 30
---------------------------
if UnitLevel("player") < 20 then btn.enabled = false end

---------------------------
-- commands
---------------------------
gf.AddCommand("Mounts","random","Summon a random mount",function() if IsMounted() then DismissCompanion("MOUNT") else gf.Mounts.summonRandom() end end)
gf.AddCommand("Mounts","random-air","Summon a random flying mount",function() if IsMounted() then DismissCompanion("MOUNT") else gf.Mounts.summonRandomAir() end end)
gf.AddCommand("Mounts","random-land","Summon a random ground mount",function() if IsMounted() then DismissCompanion("MOUNT") else gf.Mounts.summonRandomLand() end end)
gf.AddCommand("Mounts","random-water","Summon a random water mount",function() if IsMounted() then DismissCompanion("MOUNT") else gf.Mounts.summonRandomWater() end end)

---------------------------
-- variables
---------------------------
gv.setMountFavorites = false
--[[ 
if (bit.band(select(6,GetCompanionInfo("MOUNT", 22)), 0x08) == 0x08) then ... end
Ground: 0x01; Fly: 0x02; Float: 0x04; Underwater: 0x08; Jump: 0x10

Water mounts = 12
Flying, no jump, no swim = 7
Flying, no jump = 15
Flying, no swim = 23 (Red Flying Cloud and possibly others)
Ground mounts = 29 (includes swimming turtles)
Multi-purpose = 31
]]
ga.Flags = {
	--land, air, sea, swim, jump
	[7] =  {true, true, false, false, false}, -- not allowed in no fly zones?
	[12] = {false, false, true, true, false}, -- turtles, seahorses
	[15] = {false, true, true, true, false}, -- not allowed in no fly zones
	[23] = {true, true, false, false, true}, -- no swimming
	[29] = {true, false, false, true, true}, -- generally a ground mount, includes turtles
	[31] = {true, true, true, true, true}, -- special mount does everything
}	

ga.alphabet = {
	["a"] = "ae",
	["b"] = "ae",
	["c"] = "ae",
	["d"] = "ae",
	["e"] = "ae",
	["f"] = "fj",
	["g"] = "fj",
	["h"] = "fj",
	["i"] = "fj",
	["j"] = "fj",
	["k"] = "ko",
	["l"] = "ko",
	["m"] = "ko",
	["n"] = "ko",
	["o"] = "ko",
	["p"] = "pt",
	["q"] = "pt",
	["r"] = "pt",
	["s"] = "pt",
	["t"] = "pt",
	["u"] = "uz",
	["v"] = "uz",
	["w"] = "uz",
	["x"] = "uz",
	["y"] = "uz",
	["z"] = "uz",
}
ga.tmpmount = {}
ga.mount = {}

---------------------------
-- update
---------------------------
function gf.Mounts.update(self, event, arg1)
	if event == "COMPANION_LEARNED" then
		gf.Mounts.resetMountList(event)
		gf.createMenu("Mounts")
	end
end

---------------------------
-- tooltip
---------------------------
function gf.Mounts.tooltip()
	Glance.Debug("function","tooltip","Mounts")
	tooltip.Title("Mounts", "GLD")
	tooltip.Line("You have obtained "..HEX.green..GetNumCompanions("MOUNT")..HEX.white.." mounts.","WHT")
	tooltip.Space()
	tooltip.Line("Legend", "GLD")
	tooltip.Line(HEX.lightblue.."Blue |rmounts are favorites.","WHT")
	tooltip.Line(HEX.red.."Red |rmounts cannot be used in this area.","WHT")
	tooltip.Line(HEX.purple.."Purple |rmounts can not swim.","WHT")		
	tooltip.Notes("summon or dismiss a random mount",nil,"summon a specific mount","set favorites","Only favorite mounts will be summoned randomly")
end

---------------------------
-- click
---------------------------
function gf.Mounts.click(self, button, down)
	Glance.Debug("function","click","Mounts")
	if button == "LeftButton" then
		if IsMounted() then
			DismissCompanion("MOUNT")
		else
			gf.Mounts.summonRandom()
		end
	elseif button == "RightButton" then
		if IsShiftKeyDown() then
			gv.setMountFavorites = true
		else
			gv.setMountFavorites = false
		end
	end
end

---------------------------
-- onload
---------------------------
function gf.Mounts.onload()
	gf.Mounts.resetMountList()
end

---------------------------
-- menu
---------------------------
function gf.Mounts.menu(level,UIDROPDOWNMENU_MENU_VALUE)
	Glance.Debug("function","menu","Mounts")
	level = level or 1
	if (level == 1) then
		gf.setMenuTitle("Mounts Options")
		gf.setMenuHeader("Flying","flying",level)
		gf.setMenuHeader("Riding","riding",level)
		gf.setMenuHeader("Swimming","swimming",level)
	end
	if (level == 2) then
		if gf.isMenuValue("flying") then
			gf.setMenuTitle("     Flying Mounts",level, true)
			if #ga.mount > 50 then -- sort further if too many mounts
				gf.Mounts.sortMenuLines("f")
			else
				for i=1,#ga.mount do
					gf.Mounts.writeMenuLine(level,i,"f")
				end
			end
		end
		if gf.isMenuValue("riding") then
			gf.setMenuTitle("     Riding Mounts",level, true)
			if #ga.mount > 50 then -- sort further if too many mounts
				gf.Mounts.sortMenuLines("r")
			else
				for i=1,#ga.mount do
					gf.Mounts.writeMenuLine(level,i,"r")
				end
			end
		end
		if gf.isMenuValue("swimming") then
			gf.setMenuTitle("     Swimming Mounts",level, true)
			for i=1,#ga.mount do
				gf.Mounts.writeMenuLine(level,i,"s")
			end
		end
	end
	if (level == 3) then
		for i=1,#ga.mount do
			local creatureName, icon, issummoned, land, air, sea, swim, jump, enabled = unpack(ga.mount[i])
			if ga.alphabet[strlower(string.sub(creatureName,1,1))] == string.sub(UIDROPDOWNMENU_MENU_VALUE,2) then -- alpha table from name matches ui dropdown 2nd-3rd letter
				gf.Mounts.writeMenuLine(level,i,string.sub(UIDROPDOWNMENU_MENU_VALUE,1,1))
			end
		end
	end
end

---------------------------
-- sort menu line
---------------------------
function gf.Mounts.sortMenuLines(which)
	gf.setMenuHeader("Mounts A-E","ae",level)
	gf.setMenuHeader("Mounts F-J","fj",level)
	gf.setMenuHeader("Mounts K-O","ko",level)
	gf.setMenuHeader("Mounts P-T","pt",level)
	gf.setMenuHeader("Mounts U-Z","uz",level)
end

---------------------------
-- write menu line
---------------------------
function gf.Mounts.writeMenuLine(level,i,which)
	local HEX = ga.colors.HEX; local color = HEX.white;
	local creatureName, icon, issummoned, land, air, sea, swim, jump, enabled = unpack(ga.mount[i])
	if which == "f" and not air then return end -- looking for air
	if which == "r" and ((not land) or (sea or air)) then return end -- looking for land
	if which == "s" and ((not sea) or (land or air)) then return end -- looking for sea
	local ks = gf.getCondition(gv.setMountFavorites, true, false)
	local nr = gf.getCondition(gv.setMountFavorites, true, false)
	local checked, func
	if gv.setMountFavorites then -- shift used, set favorites instead of summon
		if spa.Mounts[creatureName]==true then checked = true else checked = false end
		func = function() if spa.Mounts[creatureName] then spa.Mounts[creatureName]=false; gf.Mounts.resetMountList(); else spa.Mounts[creatureName]=true; gf.Mounts.resetMountList(); end; end;
	else -- summon
		checked = gf.Mounts.isSummoned(i)
		func = function() gf.Mounts.summon(i) end;
		if not gf.Mounts.isUsable(i) then
			color = HEX.red
		elseif not swim then
			color = HEX.purple
		elseif enabled then 
			color = HEX.lightblue -- color blue others that are favorites and can be summoned
		end
	end
	--disabled,isTitle,hasArrow,notCheckable,checked,text,value,level,func,icon,keepShown,notRadio
	gf.setInfo(nil,false,false,false,checked,color..creatureName,name,level,func,icon,ks,nr)
end

---------------------------
-- can use mount here?
---------------------------
function gf.Mounts.isUsable(i)
	local creatureName, icon, issummoned, land, air, sea, swim, jump, enabled = unpack(ga.mount[i])
	if not IsOutdoors() then -- can't use any indoors
		return false;
	elseif (not jump and gf.Mounts.NoFlyZone()) then --air only in a no fly zone
		return false;
	elseif (IsSwimming() and not swim) then -- not swimmable while swimming
		return false;
	elseif strfind(creatureName,"Abyssal") and not strfind(GetMapInfo(),"Vash") then -- seahorse not in Vash
		return false;
	elseif strfind(creatureName,"Qiraji") ~= nil and not (GetZoneText() == "Ahn'Qiraj") then -- battle tank not in Qiraj
		return false;
	else
		return true;
	end
end

---------------------------
-- options
---------------------------
function gf.Mounts.options()
	local pIDX = gf.createPanel("Macros","The following macros can be made to randomize your mounts.  The choice of mount is determined by your settings in the Mounts panel.",nil)
	gf.createText(pIDX,15,-60,"/glance random","GameFontNormal","")
	gf.createText(pIDX,15,-75,"Summons a random mount best suited for the area you are in.","GameFontHighlight","")
	gf.createText(pIDX,15,-110,"/glance randomair","GameFontNormal","")
	gf.createText(pIDX,15,-125,"Summons a random flying mount.","GameFontHighlight","")
	gf.createText(pIDX,15,-160,"/glance randomland","GameFontNormal","")
	gf.createText(pIDX,15,-175,"Summons a random land mount.","GameFontHighlight","")
	gf.createText(pIDX,15,-210,"/glance randomwater","GameFontNormal","")
	gf.createText(pIDX,15,-225,"Summons a random water mount.","GameFontHighlight","")
end

---------------------------
-- is summoned
---------------------------
function gf.Mounts.isSummoned(id)
	return select(5,GetCompanionInfo("MOUNT", id))
end

---------------------------
-- summon
---------------------------
function gf.Mounts.summon(id)
	if select(5,GetCompanionInfo("MOUNT", id)) then
		DismissCompanion("MOUNT")
	else
		CallCompanion("MOUNT",id)
	end
end

---------------------------
-- get mount info
---------------------------
function gf.Mounts.mountInfo(id)
	local creatureID, creatureName, creatureSpellID, icon, issummoned, mountFlags = GetCompanionInfo("MOUNT", id);
	if mountFlags ~= nil and ga.Flags[mountFlags] ~= nil then
		local land, air, sea, swim, jump = unpack(ga.Flags[mountFlags])
		local showMount = true
		if spa.Mounts[creatureName] ~= nil then
			showMount = spa.Mounts[creatureName]
		end
		return creatureName, icon, issummoned, land, air, sea, swim, jump, showMount
	end
end

---------------------------
-- store mounts in {}
---------------------------
function gf.Mounts.resetMountList(event)
	Glance.Debug("function","resetMountList",nil)
	wipe(ga.mount)
	for i=1,GetNumCompanions("MOUNT") do
		ga.mount[i] = { gf.Mounts.mountInfo(i) }
		if spa.Mounts[ga.mount[i][1]] == nil then
			spa.Mounts[ga.mount[i][1]] = true
			if event == "COMPANION_LEARNED" then
				gf.sendMSG(ga.mount[i][1].." has been added to your mount list.  Shift-Right Click on the mount icon to disable this mount as a favorite.")
			end
		end
	end
end

---------------------------
-- no fly zones
---------------------------
function gf.Mounts.NoFlyZone()
	if not IsFlyableArea() then -- wow says no
		return true;
	end
	local pvpID, localizedName, isActive, canQueue, sec, canEnter =  (1)
	if (isActive and GetZoneText()=="Wintergrasp") then -- in battleground, wow should have said no.
		return true;
	end
	pvpID, localizedName, isActive, canQueue, sec, canEnter = GetWorldPVPAreaInfo(2)
	if (GetZoneText()=="Tol Barad") then -- in battleground, wow should have said no. isActive doesn't seem to matter here
		return true;
	end
	if GetZoneText()=="Isle of Giants" or GetZoneText()=="Isle of Thunder" or GetZoneText()=="Timeless Isle" then -- evil blizzard
		return true
	end
	return false;
end

---------------------------
-- summon a random mount
---------------------------
function gf.Mounts.summonRandom()
	Glance.Debug("function","summon","Random")
	if IsOutdoors() then
		if IsSwimming() then
			gf.Mounts.summonRandomWater()
		else
			if gf.Mounts.NoFlyZone() then
				gf.Mounts.summonRandomLand()
			else
				gf.Mounts.summonRandomAir()
			end
		end
	end
end

---------------------------
-- summon a random air mount
---------------------------
function gf.Mounts.summonRandomAir()
	local mountCount = 0
	for i=1,#ga.mount do
		local creatureName, icon, issummoned, land, air, sea, swim, jump, enabled = unpack(ga.mount[i])
		ga.mount[i][3] = false
		if air and enabled and gf.Mounts.isUsable(i) then
			mountCount = mountCount + 1
			ga.tmpmount[mountCount] = i
		end
	end
	gf.Mounts.Randomize("Air")
end

---------------------------
-- summon a random land mount
---------------------------
function gf.Mounts.summonRandomLand()
	local mountCount = 0
	if GetZoneText() == "Ahn'Qiraj" then -- in the temple, launch only battle tanks
		for i=1,#ga.mount do
			local creatureName, icon, issummoned, land, air, sea, swim, jump, enabled = unpack(ga.mount[i])
			ga.mount[i][3] = false
			if land and not air and enabled then
				if strfind(creatureName,"Qiraji") then -- allow the battle tank
					mountCount = mountCount + 1
					ga.tmpmount[mountCount] = i
				end
			end
		end
	elseif not GetZoneText() == "Ahn'Qiraj" or mountCount == 0 then -- not in the temple or no battle tanks found
		for i=1,#ga.mount do
			local creatureName, icon, issummoned, land, air, sea, swim, jump, enabled = unpack(ga.mount[i])
			ga.mount[i][3] = false
			if land and not air and enabled and (not strfind(creatureName,"Qiraji")) then
				mountCount = mountCount + 1
				ga.tmpmount[mountCount] = i
			end
		end
	end
	gf.Mounts.Randomize("Land")
end

---------------------------
-- summon a random water mount
---------------------------
function gf.Mounts.summonRandomWater()
	local mountCount = 0
	for i=1,#ga.mount do
		local creatureName, icon, issummoned, land, air, sea, swim, jump, enabled = unpack(ga.mount[i])
		ga.mount[i][3] = false
		if sea and not land and not air and enabled then
			if strfind(creatureName,"Abyssal") then -- allow the seahorse
				if strfind(GetMapInfo(),"Vash") then
					mountCount = mountCount + 1
					ga.tmpmount[mountCount] = i
					CallCompanion("MOUNT",i) -- can only be used here, just call it
					return
				end
			else
				mountCount = mountCount + 1
				ga.tmpmount[mountCount] = i
			end
		end
	end
	if mountCount==0 then -- no sea mounts available, use one that can swim
		for i=1,#ga.mount do
			local creatureName, icon, issummoned, land, air, sea, swim, jump, enabled = unpack(ga.mount[i])
			ga.mount[i][3] = false
			if swim and enabled and gf.Mounts.isUsable(i) then
				if strfind(creatureName,"Strider") then -- favor the water strider on timeless isle
					if strfind(GetMapInfo(),"Timeless") then
						mountCount = mountCount + 1
						ga.tmpmount[mountCount] = i
						CallCompanion("MOUNT",i) -- just call it
						return
					end
				end
				mountCount = mountCount + 1
				ga.tmpmount[mountCount] = i
			end
		end
	end
	gf.Mounts.Randomize("Water")
end

---------------------------
-- randomize
---------------------------
function gf.Mounts.Randomize(which)
	if #ga.tmpmount > 0 then
		local rdm = ga.tmpmount[random(1,#ga.tmpmount)]
		ga.mount[rdm][3] = true
		CallCompanion("MOUNT",rdm)
		Glance.Debug("function","summon","Random"..which..": "..ga.mount[rdm][1])
	end
	wipe(ga.tmpmount)
end

---------------------------
-- load on demand
---------------------------
if gv.loaded then
	gf.Enable("Mounts")
end